#include "../Script.hpp"
#include "../UserData/UserDataFunctionType.hpp"


namespace Porygon{
    inline bool ScriptType::CanBeIndexedWith(const ScriptType *) const{
        return false;
    }

    shared_ptr<const ScriptType> ScriptType::BoolType = make_shared<ScriptType>(TypeClass::Bool);
    shared_ptr<const ScriptType> ScriptType::NilType = make_shared<ScriptType>(TypeClass::Nil);
    shared_ptr<const ScriptType> ScriptType::AnyType = make_shared<ScriptType>(TypeClass::Any);
    shared_ptr<const NumericScriptType> NumericScriptType::AwareInt = make_shared<const NumericScriptType>(true, false);
    shared_ptr<const NumericScriptType> NumericScriptType::AwareFloat = make_shared<const NumericScriptType>(true, true);
    shared_ptr<const NumericScriptType> NumericScriptType::Unaware = make_shared<const NumericScriptType>(false, false);
    shared_ptr<const StringScriptType> StringScriptType::Dynamic = make_shared<const StringScriptType>(false,
            Utilities::HashedString::CreateLookup(0));

    shared_ptr<const ScriptType> ScriptType::GetIndexedType(shared_ptr<const ScriptType>) const{
        return make_shared<ScriptType>(TypeClass::Error);
    }

    bool ScriptType::operator==(const shared_ptr<const ScriptType> &b) const {
        if (_class == TypeClass::Nil){
            auto bClass = b->_class;
            if (bClass == TypeClass::UserData || bClass == TypeClass::String || bClass == TypeClass::Any)
                return true;
        }
        return _class == b->_class;
    }

    bool ScriptType::operator!=(const shared_ptr<const ScriptType> &b) const {
        return ! (operator==(b));
    }

    bool ScriptType::CanBeIndexedWithIdentifier(uint32_t hash) const {
        return false;
    }

    shared_ptr<const ScriptType> ScriptType::GetIndexedType(uint32_t hash) const {
        throw "This type told the binder it can be indexed, but it does not implement the resulting type.";
    }

    bool ScriptType::CanBeIterated() const {
        return false;
    }

    shared_ptr<const ScriptType> ScriptType::GetIteratorKeyType() const {
        throw "This type told the binder it can be iterated, but it does not implement the resulting type.";
    }

    CastResult ScriptType::CastableTo(const shared_ptr<const ScriptType> &castType, bool explicitCast) const {
        if (_class == TypeClass::Any){
            return CastResult ::UncheckedCast;
        }
        if (explicitCast)
            return CastResult::InvalidCast;
        return CastResult::InvalidCast;
    }

    std::string ScriptType::ToString(TypeClass c) {
        switch (c){

            case TypeClass::Error: return "error";
            case TypeClass::Nil: return "nil";
            case TypeClass::Number: return "number";
            case TypeClass::Bool: return "bool";
            case TypeClass::String: return "string";
            case TypeClass::Function: return "function";
            case TypeClass::UserData: return "userdata";
            case TypeClass::Table: return "table";
            case TypeClass::Any: return "all";
        }
        throw exception();
    }

    std::string ScriptType::ToString() const {
        return ToString(this->_class);
    }

    bool ScriptType::IsCountable() const {
        return false;
    }

    bool StringScriptType::IsCountable() const {
        return true;
    }

    extern "C"{
        ScriptType* CreateScriptType(Porygon::TypeClass t){
            return new ScriptType(t);
        }

        const ScriptType* CreateNumericScriptType(bool isAware, bool isFloat){
            return new NumericScriptType(isAware, isFloat);
        }

        ScriptType* CreateStringScriptType(bool knownAtBind, uint32_t hash){
            return new StringScriptType(knownAtBind, Utilities::HashedString::CreateLookup(hash));
        }

        ScriptType* CreateUserDataFunctionScriptType(ScriptType* returnType, ScriptType* parameters[], size_t parameterCount){
            vector<shared_ptr<const ScriptType>> vector(parameterCount);
            for (size_t i = 0; i < parameterCount; i++){
                vector[i] = shared_ptr<const ScriptType>(parameters[i]);
            }
            auto option = new UserData::UserDataFunctionOption(shared_ptr<const ScriptType>(returnType), vector);
            auto type = new GenericFunctionScriptType();
            type->RegisterFunctionOption(option);
            return type;
        }

        ScriptType* RegisterUserDataFunctionScriptTypeOption(GenericFunctionScriptType* existing, ScriptType* returnType,
                ScriptType* parameters[], size_t parameterCount){
            vector<shared_ptr<const ScriptType>> vector(parameterCount);
            for (size_t i = 0; i < parameterCount; i++){
                vector[i] = shared_ptr<const ScriptType>(parameters[i]);
            }
            auto option = new UserData::UserDataFunctionOption(shared_ptr<const ScriptType>(returnType), vector);
            existing->RegisterFunctionOption(option);
            return existing;
        }

        TypeClass GetTypeClass(ScriptType* t){
            return t->GetClass();
        }
    }
}

