
#include "UserData.hpp"
#include "UserDataStorage.hpp"

namespace Porygon::UserData {
    extern "C" {
    void RegisterUserDataType(char16_t * key) {
        auto hashedKey = new Utilities::HashedString(new u16string(key));
        auto ud = new UserData(hashedKey, {});
        UserDataStorage::RegisterType(hashedKey->GetHash(), ud);
    }

    void RegisterUserDataField(uint32_t typeId, uint32_t fieldId, UserDataField *field) {
        auto ud = UserDataStorage::GetUserDataType(typeId);
        auto get = ud->Get();
        if (get == nullptr){
            throw Exception("Userdata was not found.");
        }
        get->CreateField(fieldId, field);
    }

    int32_t GetUserDataFieldCount(uint32_t typeId) {
        auto ud = UserDataStorage::GetUserDataType(typeId);
        auto get = ud->Get();
        if (get == nullptr){
            throw Exception("Userdata was not found.");
        }
        return get->GetFieldCount();
    }

    void SetIsCastableFunc(uint32_t typeId, bool (*cast)(const ScriptType*, bool)){
        auto ud = UserDataStorage::GetUserDataType(typeId);
        auto get = ud->Get();
        if (get == nullptr){
            throw Exception("Userdata was not found.");
        }
        get->SetIsCastable(cast);
    }

    void SetCastFunc(uint32_t typeId, Evaluation::EvalValue* (*cast)(void*, const ScriptType*)){
        auto ud = UserDataStorage::GetUserDataType(typeId);
        auto get = ud->Get();
        if (get == nullptr){
            throw Exception("Userdata was not found.");
        }
        get->SetCastFunc(cast);
    }

    void AddUserdataBinaryOperation(uint32_t typeId, char kind, const UserDataBinaryOperation* operation){
        auto ud = UserDataStorage::GetUserDataType(typeId);
        auto get = ud->Get();
        if (get == nullptr){
            throw Exception("Userdata was not found.");
        }
        get->AddBinaryOperation(static_cast<Binder::BoundBinaryOperation >(kind), operation);
    }
    }
}