
#ifndef PORYGONLANG_USERDATAOPERATION_HPP
#define PORYGONLANG_USERDATAOPERATION_HPP
#include <utility>
#include "../Binder/BoundOperators.hpp"
#include "../Evaluator/EvalValues/EvalValue.hpp"

namespace Porygon::UserData {
    class UserDataBinaryOperation {
        Evaluation::EvalValue *(*_func)(void *obj, const Evaluation::EvalValue *b);
        const shared_ptr<const ScriptType> _secondParameter;
        const shared_ptr<const ScriptType> _returnType;
    public:
        UserDataBinaryOperation(Evaluation::EvalValue *(*func)(void *, const Evaluation::EvalValue *),
                                shared_ptr<const ScriptType> secondParameter,
                                shared_ptr<const ScriptType>  returnType)
                : _func(func), _secondParameter(std::move(secondParameter)),
                  _returnType(std::move(returnType)) {}

        Evaluation::EvalValue* Invoke(void* a, const Evaluation::EvalValue * b) const{
            return _func(a, b);
        }

        [[nodiscard]] bool IsValid(const shared_ptr<const ScriptType>& v) const{
            if (v->operator==(_secondParameter)){
                return true;
            }
            return v->CastableTo(_secondParameter, true) != CastResult::InvalidCast;
        }

        [[nodiscard]]
        inline shared_ptr<const ScriptType> GetReturnType() const{
            return _returnType;
        }
    };
}


#endif //PORYGONLANG_USERDATAOPERATION_HPP
